==> v11.1.0.tar.gz: brewman/brewman/brewman/routers/rate_contract.py <==
import uuid

from typing import List

from fastapi import APIRouter, HTTPException, Request, Security, status
from sqlalchemy import delete, select
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..core.session import (
    get_date,
    get_finish_date,
    get_start_date,
    set_date,
    set_period,
)
from ..db.session import SessionFuture
from ..models.rate_contract import RateContract
from ..models.rate_contract_item import RateContractItem
from ..schemas.product import ProductLink
from ..schemas.rate_contract import AccountLink
from ..schemas.rate_contract import RateContract as RateContractSchema
from ..schemas.rate_contract import RateContractBlank, RateContractIn
from ..schemas.rate_contract import RateContractItem as RateContractItemSchema
from ..schemas.user import UserToken
from ..schemas.user_link import UserLink


router = APIRouter()


@router.post("", response_model=RateContractSchema)
async def save(
    data: RateContractIn,
    request: Request,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractSchema:
    try:
        with SessionFuture() as db:
            item = RateContract(
                date=data.date_,
                vendor_id=data.vendor.id_,
                valid_from=data.valid_from,
                valid_till=data.valid_till,
                narration=data.narration,
                user_id=user.id_,
            )
            db.add(item)
            add_items(item, data.items, db)
            set_date(data.date_.strftime("%d-%b-%Y"), request.session)
            set_period(data.valid_from.strftime("%d-%b-%Y"), data.valid_till.strftime("%d-%b-%Y"), request.session)
            db.commit()
            return rate_contract_info(item)
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


def add_items(rate_contract: RateContract, items: List[RateContractItemSchema], db: Session) -> None:
    for item in items:
        rci = RateContractItem(rate_contract_id=rate_contract.id, sku_id=item.sku.id_, price=item.price)
        rate_contract.items.append(rci)
        db.add(rci)


@router.put("/{id_}", response_model=RateContractSchema)
async def update_route(
    id_: uuid.UUID,
    data: RateContractIn,
    request: Request,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractSchema:
    try:
        with SessionFuture() as db:
            item: RateContract = db.execute(select(RateContract).where(RateContract.id == id_)).scalar_one()
            item.date = data.date_
            item.vendor_id = data.vendor.id_
            item.valid_from = data.valid_from
            item.valid_till = data.valid_till
            item.narration = data.narration
            item.user_id = user.id_
            update_items(item, data.items, db)
            set_date(data.date_.strftime("%d-%b-%Y"), request.session)
            set_period(data.valid_from.strftime("%d-%b-%Y"), data.valid_till.strftime("%d-%b-%Y"), request.session)
            db.commit()
            return rate_contract_info(item)
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


def update_items(rate_contract: RateContract, items: List[RateContractItemSchema], db: Session) -> None:
    for it in range(len(rate_contract.items), 0, -1):
        item = rate_contract.items[it - 1]
        index = next((idx for (idx, d) in enumerate(items) if d.id_ == item.id), None)
        if index is not None:
            new_item = items.pop(index)
            item.sku_id = new_item.sku.id_
            item.price = new_item.price
        else:
            db.delete(item)
            rate_contract.items.remove(item)
    add_items(rate_contract, items, db)


@router.delete("/{id_}", response_model=RateContractBlank)
async def delete_route(
    id_: uuid.UUID,
    request: Request,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractBlank:
    with SessionFuture() as db:
        db.execute(delete(RateContractItem).where(RateContractItem.rate_contract_id == id_))
        db.execute(delete(RateContract).where(RateContract.id == id_))
        db.commit()
        return rate_contract_blank(request.session)


@router.get("", response_model=RateContractBlank)
async def show_blank(
    request: Request,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractBlank:
    return rate_contract_blank(request.session)


@router.get("/list", response_model=List[RateContractSchema])
async def show_list(
    user: UserToken = Security(get_user),
) -> List[RateContractSchema]:
    with SessionFuture() as db:
        return [
            rate_contract_info(item)
            for item in db.execute(select(RateContract).order_by(RateContract.date)).scalars().all()
        ]


@router.get("/{id_}", response_model=RateContractSchema)
async def show_id(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractSchema:
    with SessionFuture() as db:
        item: RateContract = db.execute(select(RateContract).where(RateContract.id == id_)).scalar_one()
        return rate_contract_info(item)


def rate_contract_info(item: RateContract) -> RateContractSchema:
    return RateContractSchema(
        id=item.id,
        date=item.date,
        vendor=AccountLink(id=item.vendor_id, name=item.vendor.name),
        validFrom=item.valid_from,
        validTill=item.valid_till,
        narration=item.narration,
        creationDate=item.creation_date,
        lastEditDate=item.last_edit_date,
        user=UserLink(id=item.user_id, name=item.user.name),
        items=[
            RateContractItemSchema(
                id=i.id,
                sku=ProductLink(
                    id=i.sku_id, name=f"{i.sku.product.name} ({i.sku.units})" if i.sku.units else i.sku.product.name
                ),
                price=i.price,
            )
            for i in item.items
        ],
    )


def rate_contract_blank(session: dict) -> RateContractBlank:
    return RateContractBlank(
        date=get_date(session),
        validFrom=get_start_date(session),
        validTill=get_finish_date(session),
        narration="",
        items=[],
    )

==> v11.1.2.tar.gz: brewman/brewman/brewman/routers/rate_contract.py <==
import uuid

from fastapi import APIRouter, HTTPException, Request, Security, status
from sqlalchemy import delete, select
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..core.session import (
    get_date,
    get_finish_date,
    get_start_date,
    set_date,
    set_period,
)
from ..db.session import SessionFuture
from ..models.rate_contract import RateContract
from ..models.rate_contract_item import RateContractItem
from ..schemas.product import ProductLink
from ..schemas.rate_contract import AccountLink
from ..schemas.rate_contract import RateContract as RateContractSchema
from ..schemas.rate_contract import RateContractBlank, RateContractIn
from ..schemas.rate_contract import RateContractItem as RateContractItemSchema
from ..schemas.user import UserToken
from ..schemas.user_link import UserLink


router = APIRouter()


@router.post("", response_model=RateContractSchema)
async def save(
    data: RateContractIn,
    request: Request,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractSchema:
    try:
        with SessionFuture() as db:
            item = RateContract(
                date_=data.date_,
                vendor_id=data.vendor.id_,
                valid_from=data.valid_from,
                valid_till=data.valid_till,
                narration=data.narration,
                user_id=user.id_,
            )
            db.add(item)
            add_items(item, data.items, db)
            set_date(data.date_, request.session)
            set_period(data.valid_from, data.valid_till, request.session)
            db.commit()
            return rate_contract_info(item)
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


def add_items(rate_contract: RateContract, items: list[RateContractItemSchema], db: Session) -> None:
    for item in items:
        rci = RateContractItem(rate_contract_id=rate_contract.id, sku_id=item.sku.id_, price=item.price)
        rate_contract.items.append(rci)
        db.add(rci)


@router.put("/{id_}", response_model=RateContractSchema)
async def update_route(
    id_: uuid.UUID,
    data: RateContractIn,
    request: Request,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractSchema:
    try:
        with SessionFuture() as db:
            item: RateContract = db.execute(select(RateContract).where(RateContract.id == id_)).scalar_one()
            item.date_ = data.date_
            item.vendor_id = data.vendor.id_
            item.valid_from = data.valid_from
            item.valid_till = data.valid_till
            item.narration = data.narration
            item.user_id = user.id_
            update_items(item, data.items, db)
            set_date(data.date_, request.session)
            set_period(data.valid_from, data.valid_till, request.session)
            db.commit()
            return rate_contract_info(item)
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


def update_items(rate_contract: RateContract, items: list[RateContractItemSchema], db: Session) -> None:
    for it in range(len(rate_contract.items), 0, -1):
        item = rate_contract.items[it - 1]
        index = next((idx for (idx, d) in enumerate(items) if d.id_ == item.id), None)
        if index is not None:
            new_item = items.pop(index)
            item.sku_id = new_item.sku.id_
            item.price = new_item.price
        else:
            db.delete(item)
            rate_contract.items.remove(item)
    add_items(rate_contract, items, db)


@router.delete("/{id_}", response_model=RateContractBlank)
async def delete_route(
    id_: uuid.UUID,
    request: Request,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractBlank:
    with SessionFuture() as db:
        db.execute(delete(RateContractItem).where(RateContractItem.rate_contract_id == id_))
        db.execute(delete(RateContract).where(RateContract.id == id_))
        db.commit()
        return rate_contract_blank(request.session)


@router.get("", response_model=RateContractBlank)
async def show_blank(
    request: Request,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractBlank:
    return rate_contract_blank(request.session)


@router.get("/list", response_model=list[RateContractSchema])
async def show_list(
    user: UserToken = Security(get_user),
) -> list[RateContractSchema]:
    with SessionFuture() as db:
        return [
            rate_contract_info(item)
            for item in db.execute(select(RateContract).order_by(RateContract.date_)).scalars().all()
        ]


@router.get("/{id_}", response_model=RateContractSchema)
async def show_id(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["rate-contracts"]),
) -> RateContractSchema:
    with SessionFuture() as db:
        item: RateContract = db.execute(select(RateContract).where(RateContract.id == id_)).scalar_one()
        return rate_contract_info(item)


def rate_contract_info(item: RateContract) -> RateContractSchema:
    return RateContractSchema(
        id_=item.id,
        date_=item.date_,
        vendor=AccountLink(id_=item.vendor_id, name=item.vendor.name),
        valid_from=item.valid_from,
        valid_till=item.valid_till,
        narration=item.narration,
        creation_date=item.creation_date,
        last_edit_date=item.last_edit_date,
        user=UserLink(id_=item.user_id, name=item.user.name),
        items=[
            RateContractItemSchema(
                id_=i.id,
                sku=ProductLink(
                    id_=i.sku_id, name=f"{i.sku.product.name} ({i.sku.units})" if i.sku.units else i.sku.product.name
                ),
                price=i.price,
            )
            for i in item.items
        ],
    )


def rate_contract_blank(session: dict) -> RateContractBlank:
    return RateContractBlank(
        date_=get_date(session),  # type: ignore[arg-type]
        valid_from=get_start_date(session),  # type: ignore[arg-type]
        valid_till=get_finish_date(session),  # type: ignore[arg-type]
        narration="",
        items=[],
    )
